/*
 * Pan - A Newsreader for X
 * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <config.h>

#include <string.h>
#include <stdlib.h>

#include <glib.h>
#include <gtk/gtk.h>

#include <pan/base/debug.h>
#include <pan/base/group.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/serverlist.h>

#include <pan/globals.h>
#include <pan/group-ui.h>
#include <pan/util.h>

static void
group_empty_cb (GtkDialog      * dialog,
		int              response,
		gpointer         groups_gp)
{
	GPtrArray * groups = (GPtrArray*) groups_gp;

	g_return_if_fail (groups!=NULL);
	g_return_if_fail (groups->len>0);

	if (response == GTK_RESPONSE_YES)
	{
		GtkWidget * w = gtk_object_get_data (GTK_OBJECT(dialog), "reset");
		guint i;
		gboolean clear_counts = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w));
		for (i=0; i<groups->len; ++i) {
			Group * group = GROUP(g_ptr_array_index(groups,i));
			group_empty (group, clear_counts);
		}
	}

	/* cleanup */
	g_ptr_array_free (groups, TRUE);
	gtk_widget_destroy (GTK_WIDGET(dialog));
}

void
group_empty_dialog (const GPtrArray * groups)
{
	GString * str = g_string_new (NULL);
	GtkWidget * dialog = NULL;
	GtkWidget * w = NULL;

	/* sanity check */
	g_return_if_fail (groups!=NULL);
	g_return_if_fail (groups->len>0);
	g_return_if_fail (group_is_valid(GROUP(g_ptr_array_index(groups,0))));

	pan_lock();

	/**
	***  Create dialog
	**/

	/* titlebar */
	if (groups->len > 1)
		g_string_sprintf (str, _("Empty %u groups"), groups->len);
	else {
		Group * g = GROUP(g_ptr_array_index(groups,0));
		g_string_sprintf (str, _("Empty group \"%s\""), group_get_readable_name(g));
	}


	dialog = gtk_dialog_new_with_buttons (str->str, GTK_WINDOW(Pan.window),
	                                      GTK_DIALOG_DESTROY_WITH_PARENT,
	                                      GTK_STOCK_NO, GTK_RESPONSE_NO,
	                                      GTK_STOCK_YES, GTK_RESPONSE_YES,
	                                      NULL);
	/* body */
	if (groups->len > 1) {
		g_string_sprintf (str,
			_("Remove all messages from these %u groups?"), groups->len);
	}
	else {
		Group * group = GROUP(g_ptr_array_index(groups,0));
		g_string_sprintf (str,
			_("Remove all messages from \"%s\"?"),
			group_get_readable_name(group));
	}
	w = gtk_label_new (str->str);
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), w, FALSE, FALSE, GUI_PAD_BIG);

	/* "clear count?" check button */
	w = gtk_check_button_new_with_label (groups->len>1
		? _("Reset \"new message\" counters")
		: _("Reset \"new message\" counter"));
	gtk_box_pack_start (GTK_BOX(GTK_DIALOG(dialog)->vbox), w, FALSE, FALSE, GUI_PAD_BIG);
	gtk_object_set_data (GTK_OBJECT(dialog), "reset", w);

	/* listen for clicks */
	g_signal_connect (GTK_OBJECT(dialog), "response",
			  G_CALLBACK (group_empty_cb), pan_g_ptr_array_dup(groups));

	gtk_widget_show_all (dialog);
	pan_unlock();

	g_string_free (str, TRUE);
}

/****
*****
*****
****/

static void
new_folder_response_cb (GtkDialog * dialog, int response, gpointer user_data)
{
	if (response == GTK_RESPONSE_OK)
	{
		Group * folder = NULL;
		Server * server = serverlist_get_named_server (INTERNAL_SERVER_NAME);
		char * name = g_strdup (gtk_entry_get_text (GTK_ENTRY(user_data)));
		debug_enter ("new_folder_cb");

		/* make sure the user entered something */
		g_strstrip (name);
		if (!is_nonempty_string (name)) {
			pan_error_dialog (_("Invalid folder name: \"%s\""), name);
			g_free (name);
			return;
		}

		/* make sure there's not already a folder with that name */
		g_strdown (name);
		folder = serverlist_get_named_folder (name);
		if (folder != NULL) {
			pan_error_dialog (_("Folder \"%s\" already exists."), name);
			g_free (name);
			return;
		}

		/* make sure it's not in the reserved namespace */
		if (!strncmp(name, "pan.", 4)) {
			pan_error_dialog (_("Please don't begin folders with \"pan.\"; it confuses me terribly."));
			g_free (name);
			return;
		}

		/* create new folder */
		folder = group_new (server, name);
		group_set_is_folder (folder, TRUE);
		group_set_dirty (folder);
		server_add_groups (server, &folder, 1, NULL, NULL);
	}

	gtk_widget_destroy (GTK_WIDGET(dialog));

	/* cleanup */
	debug_exit ("new_folder_cb");
}

void
group_new_folder_dialog (void)
{
	GtkWidget * w;
	GtkWidget * hbox;
	GtkWidget * entry;

	w = gtk_dialog_new_with_buttons (_("Create New Folder"),
	                                 GTK_WINDOW(Pan.window),
	                                 GTK_DIALOG_DESTROY_WITH_PARENT,
	                                 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
	                                 GTK_STOCK_OK, GTK_RESPONSE_OK,
	                                 NULL);
	hbox = gtk_hbox_new (FALSE, GUI_PAD);
	gtk_box_pack_start (GTK_BOX(hbox), gtk_label_new(_("New Folder Name:")),  FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX(hbox), entry = gtk_entry_new(), TRUE, TRUE, 0);
	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(w)->vbox), hbox);
	g_signal_connect (GTK_WIDGET(w), "response", G_CALLBACK(new_folder_response_cb), entry);
	gtk_widget_show_all (w);
}
